attribute highp vec3 a_Vertex;
attribute highp vec3 a_Normal;
attribute mediump vec4 a_Color;
attribute mediump vec2 a_MultiTexCoord0;
attribute mediump vec4 a_Weight;
attribute mediump vec4 a_MatrixIndex;

uniform mat4 u_modelview;      // A constant representing the combined model/view/projection matrix.
uniform mat4 u_projection;       // A constant representing the combined model/view matrix.
  
varying mediump vec4 v_Color;          // This will be passed into the fragment shader.
varying mediump vec4 v_SpecularColor;          // This will be passed into the fragment shader.
varying mediump vec2 v_texcoord;

uniform highp mat4  u_matrix_palette[9];
uniform int u_bone_count;

uniform mediump vec4 u_light_position;       // The position of the light in eye space.
uniform mediump vec3 u_light_attenuation;       // attenuation params.
uniform mediump vec4 u_ambientcolor;
uniform mediump vec4 u_lightcolor;
uniform mediump vec4 u_specular_color;
uniform mediump vec4 u_diffuse_color;
uniform mediump float u_specular_exponent;

// The entry point for our vertex shader.
void main()
{
	highp mat4 mv;
	if(u_bone_count > 0)
	{
		mediump ivec4 indexOfBone = ivec4(a_MatrixIndex);
		mediump vec4 weightOfBone = a_Weight;
		mv = u_matrix_palette[indexOfBone.x] * weightOfBone.x;
	
		for (lowp int i = 1; i < 4; i++)
		{
			if(i < u_bone_count)
			{
				indexOfBone = indexOfBone.yzwx;
				weightOfBone = weightOfBone.yzwx;
				if (weightOfBone.x > 0.0)
				{
					mv = mv + u_matrix_palette[indexOfBone.x] * weightOfBone.x;
				}
			}
		}
	}
	else
	{
		mv = u_modelview;
	}
	
	//mv = u_modelview;

    // Transform the vertex into eye space.
    highp vec3 pos = vec3(mv * vec4(a_Vertex, 1.0));
 
    // Pass through the color.
    v_Color = a_Color;
  
    // Transform the normal's orientation into eye space.
    mediump vec3 nrm = normalize(vec3(mv * vec4(a_Normal, 0.0)));
 
    gl_Position = u_projection * mv * vec4(a_Vertex, 1.0);
    
	v_texcoord = a_MultiTexCoord0;
	
	mediump vec4 final_color;
 
	mediump vec3 lightVector;
	
	if(u_light_position.w == 1.0)
		lightVector = normalize(u_light_position.xyz - pos);    
	else
		lightVector = u_light_position.xyz;
 
	// Calculate the dot product of the light vector and vertex normal. If the normal and light vector are
	// pointing in the same direction then it will get max illumination.
	mediump float diffuse = 0.0;
	
	if(u_diffuse_color.xyz != vec3(0.0, 0.0, 0.0))
	{
		diffuse = max(dot(nrm, lightVector), 0.0);
	
		if(u_light_position.w == 1.0)
		{
			lowp float distance = length(u_light_position.xyz - pos);
			diffuse = diffuse * (1.0 / (u_light_attenuation.x + u_light_attenuation.y * distance + u_light_attenuation.z * distance * distance));
		}
			
		diffuse = min(diffuse, 1.0);
	}
    
	mediump vec4 specular = vec4(0, 0, 0, 0);
    
	if(u_specular_exponent > 0.001)
	{
		mediump vec3 halfVec = normalize((lightVector + vec3(0.0, 0.0, 1.0)) / 2.0);
		lowp float highlights = max(dot(nrm, halfVec), 0.0);
		specular = u_lightcolor * pow(highlights, u_specular_exponent) * u_specular_color;
	}
 
	// Multiply the color by the diffuse illumination level to get final output color.
    
	final_color = (v_Color * diffuse * u_lightcolor + u_ambientcolor * v_Color) * u_diffuse_color;
		
    final_color.a = a_Color.a;
    
    v_Color = final_color;
    v_SpecularColor = specular;
}


