uniform mat4 u_modelview;      // A constant representing the combined model/view/projection matrix.
uniform mat4 u_projection;       // A constant representing the combined model/view matrix.

uniform vec3 u_object_center;
  
varying vec4 v_Color;          // This will be passed into the fragment shader.

varying vec2 v_texcoord;

uniform vec4 u_light_position;       // The position of the light in eye space.
uniform vec3 u_light_attenuation;       // attenuation params.
uniform vec4 u_ambientcolor;
uniform vec4 u_lightcolor;
uniform vec4 u_diffuse_color;

attribute vec3 a_Vertex;
attribute vec3 a_Normal;
attribute mediump vec4 a_Color;
attribute mediump vec2 a_MultiTexCoord0;

 
// The entry point for our vertex shader.
void main()
{
	gl_Position = u_projection * u_modelview * vec4(a_Vertex, 1.0);

	v_texcoord = a_MultiTexCoord0.xy;

    // Transform the vertex into eye space.
    vec3 pos = vec3(u_modelview * vec4(a_Vertex, 1.0));
 
    // Pass through the color.
    v_Color = a_Color;

	vec3 nrm = a_Vertex - u_object_center;
	nrm.y *= 0.2;
 	nrm = normalize(nrm); 
 
    nrm = normalize(vec3(u_modelview * vec4(nrm, 0.0)));
    
    vec4 final_color;
 
	vec3 lightVector;
	
	if(u_light_position.w == 1.0)
		lightVector = normalize(u_light_position.xyz - pos);    
	else
		lightVector = u_light_position.xyz;
 
	// Calculate the dot product of the light vector and vertex normal. If the normal and light vector are
	// pointing in the same direction then it will get max illumination.
	float diffuse = 0.0;
	
	if(u_diffuse_color.xyz != vec3(0.0, 0.0, 0.0))
	{
		diffuse = max(dot(nrm, lightVector), 0.0);
	
		if(u_light_position.w == 1.0)
		{
			float distance = length(u_light_position.xyz - pos );
			diffuse = diffuse * (1.0 / (u_light_attenuation.x + u_light_attenuation.y * distance + u_light_attenuation.z * distance * distance));
		} 
		
		diffuse = min(diffuse, 1.0);
		//diffuse *= 0.8;
	}
       
	final_color = v_Color * (diffuse * u_lightcolor * u_diffuse_color + u_ambientcolor * 0.6);
	final_color.r = clamp(final_color.r, 0.0, 1.0);
	final_color.g = clamp(final_color.g, 0.0, 1.0);
	final_color.b = clamp(final_color.b, 0.0, 1.0);
	
    final_color.a = v_Color.a;
    
    v_Color = final_color; 
}


